
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <linux/fs.h>

#include "vol.h"

char *usage =
"Usage: %s [device] [type] [devices]\n\n"
"Set up a volume device according to the arguments\n\n"
"  device       volume device, e.g. /dev/vol0\n"
"  type         volume type, only 'linear' and 'stripe' are supported for now\n"
"  devices      list of devices\n";

int
main (int argc, char **argv)
{
  char *volume;
  int volumef;
  int type;
  struct stat statbuf;
  int x;
  int devf;
  char *dev;
  long devsize;
  struct vol_add vadd;

  if (argc < 4)
    {
      if (argc == 2)
	if (!strcmp (argv[1], "--help") || !strcmp (argv[1], "-h"))
	  {
	    printf (usage, argv[0]);
	    return 0;
	  }
      fprintf (stderr, "Too few arguments\n");
      printf (usage, argv[0]);
      return -1;
    }

  volume = argv[1];
  if (!strcmp (argv[2], "linear"))
    type = VOL_LINEAR;
  else if (!strcmp (argv[2], "stripe"))
    type = VOL_STRIPE;
  else
    {
      fprintf (stderr, "%s: unknown volume type '%s'\n", volume, argv[2]);
      printf (usage, argv[0]);
      return -1;
    }

  volumef = open (volume, O_RDONLY);
  if (volumef == -1)
    {
      perror (volume);
      return -1;
    }

  if (fstat (volumef, &statbuf) != 0)
    {
      perror (volume);
      return -1;
    }

  if (!S_ISBLK (statbuf.st_mode))
    {
      fprintf (stderr, "%s: Not a block device\n", volume);
      return -1;
    }

  if (ioctl (volumef, VOL_IOCTL_SETUP, &type) == -1)
    {
      perror (volume);
      return -1;
    }

  for (x = 3; x < argc; x++)
    {
      dev = argv[x];
      printf ("Adding device %s\n", dev);
      devf = open (dev, O_RDONLY);
      if (devf == -1)
	{
	  perror (dev);
	  return -1;
	}
      if (fstat (devf, &statbuf) != 0)
	{
	  perror (dev);
	  return -1;
	}

      if (!S_ISBLK (statbuf.st_mode))
	{
	  fprintf (stderr, "%s: Not a block device\n", dev);
	  return -1;
	}
      if (ioctl (devf, BLKGETSIZE, &devsize) != 0)
	{
	  perror (dev);
	  return -1;
	}
      close (devf);

      vadd.dev = statbuf.st_rdev;
      vadd.size = devsize * 512;	// FIXME!! - sector size 512

      if (ioctl (volumef, VOL_IOCTL_ADD, &vadd) == -1)
	{
	  perror (volume);
	  return -1;
	}

    }

  if (ioctl (volumef, VOL_IOCTL_START, 0) == -1)
    {
      perror (volume);
      return -1;
    }

  close (volumef);
  return 0;
}
